@@ -24,7 +24,6 @@ ac_table = {
             '--index-labels-url',
             '--query-data-url',
             '--query-labels-url',
-            '--request-size',
             '--num-query',
             '--top-k',
         ],
@@ -51,11 +50,14 @@ ac_table = {
             '--help',
             '--name',
             '--workspace',
+            '--k8s-namespace',
             '--log-config',
             '--quiet',
             '--quiet-error',
             '--identity',
             '--workspace-id',
+            '--static-routing-table',
+            '--routing-table',
             '--zmq-identity',
             '--port-ctrl',
             '--ctrl-with-ipc',
@@ -83,14 +85,16 @@ ac_table = {
             '--dynamic-routing-in',
             '--grpc-data-requests',
             '--runs-in-docker',
+            '--dump-path',
+            '--k8s-connection-pool',
+            '--k8s-no-connection-pool',
             '--entrypoint',
             '--docker-kwargs',
             '--pull-latest',
             '--volumes',
             '--gpus',
             '--host',
-            '--port-expose',
-            '--proxy',
+            '--port-jinad',
             '--quiet-remote-logs',
             '--upload-files',
             '--disable-remote',
@@ -104,29 +108,38 @@ ac_table = {
             '--pea-id',
             '--pea-role',
             '--noblock-on-start',
+            '--install-requirements',
+            '--force',
         ],
         'flow': [
             '--help',
             '--name',
             '--workspace',
+            '--k8s-namespace',
             '--log-config',
             '--quiet',
             '--quiet-error',
             '--workspace-id',
+            '--static-routing-table',
+            '--routing-table',
             '--uses',
             '--env',
             '--inspect',
+            '--infrastructure',
         ],
         'ping': ['--help', '--timeout', '--retries', '--print-response'],
         'gateway': [
             '--help',
             '--name',
             '--workspace',
+            '--k8s-namespace',
             '--log-config',
             '--quiet',
             '--quiet-error',
             '--identity',
             '--workspace-id',
+            '--static-routing-table',
+            '--routing-table',
             '--zmq-identity',
             '--port-ctrl',
             '--ctrl-with-ipc',
@@ -154,6 +167,9 @@ ac_table = {
             '--dynamic-routing-in',
             '--grpc-data-requests',
             '--runs-in-docker',
+            '--dump-path',
+            '--k8s-connection-pool',
+            '--k8s-no-connection-pool',
             '--prefetch',
             '--prefetch-on-recv',
             '--title',
@@ -163,13 +179,14 @@ ac_table = {
             '--no-debug-endpoints',
             '--no-crud-endpoints',
             '--expose-endpoints',
+            '--uvicorn-kwargs',
             '--compress',
             '--compress-min-bytes',
             '--compress-min-ratio',
             '--protocol',
             '--host',
-            '--port-expose',
             '--proxy',
+            '--port-expose',
             '--daemon',
             '--runtime-backend',
             '--runtime',
@@ -180,33 +197,47 @@ ac_table = {
             '--pea-id',
             '--pea-role',
             '--noblock-on-start',
-            '--routing-table',
+            '--k8s-uses-init',
+            '--k8s-mount-path',
+            '--k8s-init-container-command',
+            '--k8s-custom-resource-dir',
             '--dynamic-routing',
-            '--connect-to-predecessor',
         ],
-        'hub new': ['--help'],
+        'hub new': [
+            '--help',
+            '--name',
+            '--path',
+            '--advance-configuration',
+            '--description',
+            '--keywords',
+            '--url',
+            '--add-dockerfile',
+        ],
         'hub push': [
             '--help',
             '--no-usage',
-            '--docker-file',
+            '--dockerfile',
             '--tag',
             '--force',
             '--secret',
             '--public',
             '--private',
         ],
-        'hub pull': ['--help', '--no-usage', '--install-requirements'],
+        'hub pull': ['--help', '--no-usage', '--install-requirements', '--force'],
         'hub': ['--help', 'new', 'push', 'pull'],
         'help': ['--help'],
         'pea': [
             '--help',
             '--name',
             '--workspace',
+            '--k8s-namespace',
             '--log-config',
             '--quiet',
             '--quiet-error',
             '--identity',
             '--workspace-id',
+            '--static-routing-table',
+            '--routing-table',
             '--zmq-identity',
             '--port-ctrl',
             '--ctrl-with-ipc',
@@ -234,14 +265,16 @@ ac_table = {
             '--dynamic-routing-in',
             '--grpc-data-requests',
             '--runs-in-docker',
+            '--dump-path',
+            '--k8s-connection-pool',
+            '--k8s-no-connection-pool',
             '--entrypoint',
             '--docker-kwargs',
             '--pull-latest',
             '--volumes',
             '--gpus',
             '--host',
-            '--port-expose',
-            '--proxy',
+            '--port-jinad',
             '--quiet-remote-logs',
             '--upload-files',
             '--disable-remote',
@@ -255,16 +288,21 @@ ac_table = {
             '--pea-id',
             '--pea-role',
             '--noblock-on-start',
+            '--install-requirements',
+            '--force',
         ],
         'pod': [
             '--help',
             '--name',
             '--workspace',
+            '--k8s-namespace',
             '--log-config',
             '--quiet',
             '--quiet-error',
             '--identity',
             '--workspace-id',
+            '--static-routing-table',
+            '--routing-table',
             '--zmq-identity',
             '--port-ctrl',
             '--ctrl-with-ipc',
@@ -292,14 +330,16 @@ ac_table = {
             '--dynamic-routing-in',
             '--grpc-data-requests',
             '--runs-in-docker',
+            '--dump-path',
+            '--k8s-connection-pool',
+            '--k8s-no-connection-pool',
             '--entrypoint',
             '--docker-kwargs',
             '--pull-latest',
             '--volumes',
             '--gpus',
             '--host',
-            '--port-expose',
-            '--proxy',
+            '--port-jinad',
             '--quiet-remote-logs',
             '--upload-files',
             '--disable-remote',
@@ -313,6 +353,8 @@ ac_table = {
             '--pea-id',
             '--pea-role',
             '--noblock-on-start',
+            '--install-requirements',
+            '--force',
             '--uses-before',
             '--uses-after',
             '--parallel',
@@ -325,12 +367,17 @@ ac_table = {
             '--pod-role',
             '--no-dynamic-routing',
             '--connect-to-predecessor',
+            '--k8s-uses-init',
+            '--k8s-mount-path',
+            '--k8s-init-container-command',
+            '--k8s-custom-resource-dir',
         ],
         'client': [
             '--help',
             '--host',
-            '--port-expose',
             '--proxy',
+            '--port',
+            '--https',
             '--asyncio',
             '--protocol',
         ],
