@@ -1,7 +1,7 @@
 from fastapi import Depends, APIRouter, HTTPException
 
-from ..dependencies import PeaDepends
 from ... import Runtime400Exception
+from ..dependencies import PeaDepends
 from ...models import DaemonID, ContainerItem, ContainerStoreStatus, PeaModel
 from ...stores import pea_store as store
 
@@ -37,6 +37,8 @@ async def _create(pea: PeaDepends = Depends(PeaDepends)):
             workspace_id=pea.workspace_id,
             params=pea.params,
             ports=pea.ports,
+            envs=pea.envs,
+            device_requests=pea.device_requests,
         )
     except Exception as ex:
         raise Runtime400Exception from ex
@@ -61,7 +63,7 @@ async def _delete(id: DaemonID, workspace: bool = False):
     try:
         await store.delete(id=id, workspace=workspace)
     except KeyError:
-        raise HTTPException(status_code=404, detail=f'{id} not found in {store!r}')
+        raise HTTPException(status_code=404, detail=f'{id} not found in store')
 
 
 @router.get(
