@@ -4,18 +4,23 @@ import asyncio
 from copy import deepcopy
 from platform import uname
 from http import HTTPStatus
-from typing import Dict, TYPE_CHECKING
+from typing import Dict, TYPE_CHECKING, List, Optional, Union
 
 import aiohttp
 
 from jina import __docker_host__
 from jina.helper import colored, random_port
+from jina.enums import RemoteWorkspaceState
 from .base import BaseStore
 from ..dockerize import Dockerizer
-from ..excepts import PartialDaemon400Exception, Runtime400Exception
-from ..helper import id_cleaner
+from ..excepts import (
+    PartialDaemon400Exception,
+    PartialDaemonConnectionException,
+)
+from ..helper import if_alive, id_cleaner, error_msg_from
 from ..models import DaemonID
-from ..models.enums import UpdateOperation
+from ..models.ports import PortMappings
+from ..models.enums import UpdateOperation, IDLiterals
 from ..models.containers import (
     ContainerArguments,
     ContainerItem,
@@ -34,28 +39,44 @@ class ContainerStore(BaseStore):
     _status_model = ContainerStoreStatus
 
     async def _add(self, uri, *args, **kwargs):
-        """Implements jina object creation in `mini-jinad`
+        """Implements jina object creation in `partial-daemon`
 
         .. #noqa: DAR101"""
         raise NotImplementedError
 
-    async def _update(self, uri, *args, **kwargs):
-        """Implements jina object update in `mini-jinad`
+    @if_alive
+    async def _update(self, uri: str, params: Dict, **kwargs) -> Dict:
+        """Sends `PUT` request to `partial-daemon` to execute a command on a Flow.
 
-        .. #noqa: DAR101"""
-        raise NotImplementedError
+        :param uri: uri of partial-daemon
+        :param params: json payload to be sent
+        :param kwargs: keyword args
+        :raises PartialDaemon400Exception: if update fails
+        :return: response from partial-daemon
+        """
+
+        self._logger.debug(
+            f'sending PUT request to partial-daemon on {uri}/{self._kind}'
+        )
+        async with aiohttp.request(
+            method='PUT', url=f'{uri}/{self._kind}', params=params
+        ) as response:
+            response_json = await response.json()
+            if response.status != HTTPStatus.OK:
+                raise PartialDaemon400Exception(error_msg_from(response_json))
+            return response_json
 
     async def _delete(self, uri, *args, **kwargs):
-        """Implements jina object termination in `mini-jinad`
+        """Implements jina object termination in `partial-daemon`
 
         .. #noqa: DAR101"""
         raise NotImplementedError
 
     async def ready(self, uri) -> bool:
-        """Check if the container with mini-jinad is alive
+        """Check if the container with partial-daemon is alive
 
-        :param uri: uri of mini-jinad
-        :return: True if mini-jinad is ready"""
+        :param uri: uri of partial-daemon
+        :return: True if partial-daemon is ready"""
         async with aiohttp.ClientSession() as session:
             for _ in range(20):
                 try:
@@ -70,7 +91,7 @@ class ContainerStore(BaseStore):
                     continue
                 except Exception as e:
                     self._logger.error(
-                        f'error while checking if mini-jinad is ready: {e}'
+                        f'error while checking if partial-daemon is ready: {e}'
                     )
         self._logger.error(
             f'couldn\'t reach {self._kind.title()} container at {uri} after 10secs'
@@ -78,7 +99,7 @@ class ContainerStore(BaseStore):
         return False
 
     def _uri(self, port: int) -> str:
-        """Returns uri of mini-jinad.
+        """Returns uri of partial-daemon.
 
         NOTE: JinaD (running inside a container) needs to access other containers via dockerhost.
         Mac/WSL: this would work as is, as dockerhost is accessible.
@@ -87,7 +108,7 @@ class ContainerStore(BaseStore):
         NOTE: Checks if we actually are in docker (needed for unit tests). If not docker, use localhost.
 
         :param port: mini jinad port
-        :return: uri for mini-jinad
+        :return: uri for partial-daemon
         """
 
         if (
@@ -99,17 +120,18 @@ class ContainerStore(BaseStore):
         else:
             return f'http://{__docker_host__}:{port}'
 
-    def _command(self, port: int, workspace_id: DaemonID) -> str:
-        """Returns command for mini-jinad container to be appended to default entrypoint
+    def _entrypoint(self, port: int, workspace_id: DaemonID) -> str:
+        """Returns entrypoint for partial-daemon container to be appended to default entrypoint
 
-        NOTE: `command` is appended to already existing entrypoint, hence removed the prefix `jinad`
         NOTE: Important to set `workspace_id` here as this gets set in jina objects in the container
 
-        :param port: mini jinad port
+        :param port: partial-daemon port
         :param workspace_id: workspace id
-        :return: command for mini-jinad container
+        :return: command for partial-daemon container
         """
-        return f'--port-expose {port} --mode {self._kind} --workspace-id {workspace_id.jid}'
+        return (
+            f'jinad --port {port} --mode {self._kind} --workspace-id {workspace_id.jid}'
+        )
 
     @BaseStore.dump
     async def add(
@@ -117,7 +139,9 @@ class ContainerStore(BaseStore):
         id: DaemonID,
         workspace_id: DaemonID,
         params: 'BaseModel',
-        ports: Dict,
+        ports: Union[Dict, PortMappings],
+        envs: Optional[Dict[str, str]] = {},
+        device_requests: Optional[List] = None,
         **kwargs,
     ) -> DaemonID:
         """Add a container to the store
@@ -126,9 +150,11 @@ class ContainerStore(BaseStore):
         :param workspace_id: workspace id where the container lives
         :param params: pydantic model representing the args for the container
         :param ports: ports to be mapped to local
+        :param envs: dict of env vars to be passed
+        :param device_requests: docker device requests
         :param kwargs: keyword args
-        :raises KeyError: if workspace_id doesn't exist in the store
-        :raises Runtime400Exception: if container creation fails
+        :raises KeyError: if workspace_id doesn't exist in the store or not ACTIVE
+        :raises PartialDaemonConnectionException: if jinad cannot connect to partial
         :return: id of the container
         """
         try:
@@ -136,11 +162,18 @@ class ContainerStore(BaseStore):
 
             if workspace_id not in workspace_store:
                 raise KeyError(f'{workspace_id} not found in workspace store')
+            elif workspace_store[workspace_id].state != RemoteWorkspaceState.ACTIVE:
+                raise KeyError(
+                    f'{workspace_id} is not ACTIVE yet. Please retry once it becomes ACTIVE'
+                )
 
-            minid_port = random_port()
-            ports.update({f'{minid_port}/tcp': minid_port})
-            uri = self._uri(minid_port)
-            command = self._command(minid_port, workspace_id)
+            partiald_port = random_port()
+            dockerports = (
+                ports.docker_ports if isinstance(ports, PortMappings) else ports
+            )
+            dockerports.update({f'{partiald_port}/tcp': partiald_port})
+            uri = self._uri(partiald_port)
+            entrypoint = self._entrypoint(partiald_port, workspace_id)
             params = params.dict(exclude={'log_config'})
 
             self._logger.debug(
@@ -149,26 +182,36 @@ class ContainerStore(BaseStore):
                     [
                         '{:15s} -> {:15s}'.format('id', id),
                         '{:15s} -> {:15s}'.format('workspace', workspace_id),
-                        '{:15s} -> {:15s}'.format('ports', str(ports)),
-                        '{:15s} -> {:15s}'.format('command', command),
+                        '{:15s} -> {:15s}'.format('dockerports', str(dockerports)),
+                        '{:15s} -> {:15s}'.format('entrypoint', entrypoint),
                     ]
                 )
             )
 
-            container, network, ports = Dockerizer.run(
-                workspace_id=workspace_id, container_id=id, command=command, ports=ports
+            container, network, dockerports = Dockerizer.run(
+                workspace_id=workspace_id,
+                container_id=id,
+                entrypoint=entrypoint,
+                ports=dockerports,
+                envs=envs,
+                device_requests=device_requests,
             )
             if not await self.ready(uri):
-                raise Runtime400Exception(
+                raise PartialDaemonConnectionException(
                     f'{id.type.title()} creation failed, couldn\'t reach the container at {uri} after 10secs'
                 )
+            kwargs.update(
+                {'ports': ports.dict()} if isinstance(ports, PortMappings) else {}
+            )
             object = await self._add(uri=uri, params=params, **kwargs)
         except Exception as e:
             self._logger.error(f'{self._kind} creation failed as {e}')
             container_logs = Dockerizer.logs(container.id)
-            if container_logs and isinstance(e, PartialDaemon400Exception):
+            if container_logs and isinstance(
+                e, (PartialDaemon400Exception, PartialDaemonConnectionException)
+            ):
                 self._logger.debug(
-                    f'error logs frm partial daemon: \n {container_logs}'
+                    f'error logs from partial daemon: \n {container_logs}'
                 )
                 if e.message and isinstance(e.message, list):
                     e.message += container_logs.split('\n')
@@ -185,11 +228,11 @@ class ContainerStore(BaseStore):
                     container_name=container.name,
                     image_id=id_cleaner(container.image.id),
                     network=network,
-                    ports=ports,
+                    ports=dockerports,
                     uri=uri,
                 ),
                 arguments=ContainerArguments(
-                    command=command,
+                    entrypoint=entrypoint,
                     object=object,
                 ),
                 workspace_id=workspace_id,
@@ -224,12 +267,18 @@ class ContainerStore(BaseStore):
         if id not in self:
             raise KeyError(f'{colored(id, "red")} not found in store.')
 
-        params = {
-            'kind': kind.value,
-            'dump_path': dump_path,
-            'pod_name': pod_name,
-        }
-        params.update({'shards': shards} if shards else {})
+        if id.jtype == IDLiterals.JFLOW:
+            params = {
+                'kind': kind.value,
+                'dump_path': dump_path,
+                'pod_name': pod_name,
+            }
+            params.update({'shards': shards} if shards else {})
+        elif id.jtype == IDLiterals.JPOD:
+            params = {'kind': kind.value, 'dump_path': dump_path}
+        else:
+            self._logger.error(f'update not supported for {id.type} {id}')
+            return id
 
         uri = self[id].metadata.uri
         try:
