@@ -7,11 +7,12 @@ import os
 import shelve
 import subprocess
 import sys
+import tarfile
 import urllib
 import zipfile
 from functools import lru_cache, wraps
 from pathlib import Path
-from typing import Tuple, Optional, Dict
+from typing import Tuple, Optional, Dict, List
 from urllib.parse import urlparse, urljoin
 from urllib.request import Request, urlopen
 
@@ -105,7 +106,7 @@ def unpack_package(filepath: 'Path', target_dir: 'Path'):
         with zipfile.ZipFile(filepath, 'r') as zip:
             zip.extractall(target_dir)
     elif filepath.suffix in ['.tar', '.gz']:
-        with zipfile.open(filepath) as tar:
+        with tarfile.open(filepath) as tar:
             tar.extractall(target_dir)
     else:
         raise ValueError('File format is not supported for unpacking.')
@@ -263,6 +264,8 @@ def disk_cache_offline(
 ):
     """
     Decorator which caches a function in disk and uses cache when a urllib.error.URLError exception is raised
+    If the function was called with a kwarg force=True, then this decorator will always attempt to call it, otherwise,
+    will always default to local cache.
 
     :param cache_file: the cache file
     :param message: the warning message shown when defaulting to cache. Use "{func_name}" if you want to print
@@ -275,14 +278,34 @@ def disk_cache_offline(
         @wraps(func)
         def wrapper(*args, **kwargs):
             call_hash = f'{func.__name__}({", ".join(map(str, args))})'
-            with shelve.open(cache_file) as cache_db:
+
+            pickle_protocol = 4
+
+            try:
+                cache_db = shelve.open(
+                    cache_file, protocol=pickle_protocol, writeback=True
+                )
+            except Exception as ex:
+                if os.path.exists(cache_file):
+                    # cache is in an unsupported format, reset the cache
+                    os.remove(cache_file)
+                    cache_db = shelve.open(
+                        cache_file, protocol=pickle_protocol, writeback=True
+                    )
+                else:
+                    raise
+
+            with cache_db as dict_db:
                 try:
+                    if call_hash in dict_db and not kwargs.get('force', False):
+                        return dict_db[call_hash]
+
                     result = func(*args, **kwargs)
-                    cache_db[call_hash] = result
+                    dict_db[call_hash] = result
                 except urllib.error.URLError:
-                    if call_hash in cache_db:
+                    if call_hash in dict_db:
                         default_logger.warning(message.format(func_name=func.__name__))
-                        return cache_db[call_hash]
+                        return dict_db[call_hash]
                     else:
                         raise
             return result
@@ -292,11 +315,34 @@ def disk_cache_offline(
     return decorator
 
 
-def install_requirements(requirements_file: 'Path'):
+def install_requirements(
+    requirements_file: 'Path', timeout: int = 1000, excludes: Tuple[str] = ('jina',)
+):
     """Install modules included in requirments file
-
     :param requirements_file: the requirements.txt file
+    :param timeout: the socket timeout (default = 1000s)
+    :param excludes: the excluded module dependencies
     """
+    import pkg_resources
+
+    with requirements_file.open() as requirements:
+        install_reqs = [
+            str(req)
+            for req in pkg_resources.parse_requirements(requirements)
+            if req.project_name not in excludes or len(req.extras) > 0
+        ]
+
+    if len(install_reqs) == 0:
+        return
+
     subprocess.check_call(
-        [sys.executable, '-m', 'pip', 'install', '-r', f'{requirements_file}']
+        [
+            sys.executable,
+            '-m',
+            'pip',
+            'install',
+            '--compile',
+            f'--default-timeout={timeout}',
+        ]
+        + install_reqs
     )
