@@ -2,6 +2,9 @@ import os
 import copy
 import asyncio
 import argparse
+from typing import TYPE_CHECKING, Union
+
+from ....enums import SocketType
 
 from ...zmq import send_ctrl_message
 from ....jaml.helper import complete_path
@@ -14,17 +17,27 @@ from ....excepts import (
     DaemonWorkspaceCreationFailed,
 )
 
+if TYPE_CHECKING:
+    import multiprocessing
+    import threading
+    from ....logging.logger import JinaLogger
+
 
 class JinadRuntime(AsyncNewLoopRuntime):
     """Runtime procedure for JinaD."""
 
     def __init__(
-        self, args: 'argparse.Namespace', ctrl_addr: str, timeout_ctrl: int, **kwargs
+        self,
+        args: 'argparse.Namespace',
+        **kwargs,
     ):
-        super().__init__(args, ctrl_addr, **kwargs)
+        super().__init__(args, **kwargs)
         # Need the `proper` control address to send `activate` and `deactivate` signals, from the pea in the `main`
         # process.
-        self.timeout_ctrl = timeout_ctrl
+        self.ctrl_addr = self.get_control_address(args.host, args.port_ctrl)
+        self.timeout_ctrl = args.timeout_ctrl
+        self.host = args.host
+        self.port_jinad = args.port_jinad
 
     async def async_setup(self):
         """Create Workspace, Pea on remote JinaD server"""
@@ -33,7 +46,6 @@ class JinadRuntime(AsyncNewLoopRuntime):
             import rich
             import aiohttp
             from daemon.clients import AsyncJinaDClient
-            from daemon.models import DaemonID
 
             assert rich
             assert aiohttp
@@ -41,7 +53,7 @@ class JinadRuntime(AsyncNewLoopRuntime):
         # NOTE: args.timeout_ready is always set to -1 for JinadRuntime so that wait_for_success doesn't fail in Pea,
         # so it can't be used for Client timeout.
         self.client = AsyncJinaDClient(
-            host=self.args.host, port=self.args.port_expose, logger=self.logger
+            host=self.args.host, port=self.args.port_jinad, logger=self.logger
         )
         if not await self.client.alive:
             raise DaemonConnectivityError
@@ -66,18 +78,15 @@ class JinadRuntime(AsyncNewLoopRuntime):
             raise DaemonPeaCreationFailed
 
     async def _wait_for_cancel(self):
-        """Do NOT override this method when inheriting from :class:`GatewayPea`"""
-        while True:
-            if self.is_cancel.is_set():
-                await self.async_cancel()
-                send_ctrl_message(self.ctrl_addr, 'TERMINATE', self.timeout_ctrl)
-                return
-            else:
-                await asyncio.sleep(0.1)
+        while not self.is_cancel.is_set():
+            await asyncio.sleep(0.1)
+
+        send_ctrl_message(self.ctrl_addr, 'TERMINATE', self.timeout_ctrl)
 
     async def async_run_forever(self):
-        """Streams log messages using websocket from remote server"""
-        self.is_ready_event.set()
+        """
+        Streams log messages using websocket from remote server
+        """
         self.logstream = asyncio.create_task(
             self._sleep_forever()
             if self.args.quiet_remote_logs
@@ -85,11 +94,12 @@ class JinadRuntime(AsyncNewLoopRuntime):
         )
 
     async def async_cancel(self):
-        """Cancels the logstream task, removes the remote Pea & Workspace"""
+        """Cancels the logstream task, removes the remote Pea"""
         self.logstream.cancel()
-        await self.client.peas.delete(id=self.pea_id)
-        # NOTE: don't fail if workspace deletion fails here
-        await self.client.workspaces.delete(id=self.workspace_id)
+        if await self.client.peas.delete(id=self.pea_id):
+            self.logger.success(f'Successfully terminated remote Pea {self.pea_id}')
+        # Don't delete workspace here, as other Executors might use them.
+        # TODO(Deepankar): probably enable an arg here?
 
     async def _sleep_forever(self):
         """Sleep forever, no prince will come."""
@@ -98,13 +108,15 @@ class JinadRuntime(AsyncNewLoopRuntime):
     def _mask_args(self, args: 'argparse.Namespace'):
         _args = copy.deepcopy(args)
 
-        # reset the runtime to ZEDRuntime or ContainerRuntime
+        # reset the runtime to ZEDRuntime/GRPCDataRuntime or ContainerRuntime
         if _args.runtime_cls == 'JinadRuntime':
-            # TODO: add jinahub:// and jinahub+docker:// scheme here
-            if _args.uses.startswith('docker://'):
+            if _args.uses.startswith(('docker://', 'jinahub+docker://')):
                 _args.runtime_cls = 'ContainerRuntime'
             else:
-                _args.runtime_cls = 'ZEDRuntime'
+                if _args.grpc_data_requests:
+                    _args.runtime_cls = 'GRPCDataRuntime'
+                else:
+                    _args.runtime_cls = 'ZEDRuntime'
 
         # TODO:/NOTE this prevents jumping from remote to another remote (Han: 2021.1.17)
         # _args.host = __default_host__
@@ -113,7 +125,7 @@ class JinadRuntime(AsyncNewLoopRuntime):
 
         # NOTE: on remote relative filepaths should be converted to filename only
         def basename(field):
-            if field and not field.startswith('docker://'):
+            if field and not field.startswith(('docker://', 'jinahub')):
                 try:
                     return os.path.basename(complete_path(field))
                 except FileNotFoundError:
@@ -143,3 +155,82 @@ class JinadRuntime(AsyncNewLoopRuntime):
             self.logger.debug('\n'.join(changes))
 
         return _args
+
+    # Static methods used by the Pea to communicate with the `Runtime` in the separate process
+
+    @staticmethod
+    def cancel(
+        cancel_event: Union['multiprocessing.Event', 'threading.Event'], **kwargs
+    ):
+        """
+        Signal the runtime to terminate
+
+        :param cancel_event: the cancel event to set
+        :param kwargs: extra keyword arguments
+        """
+        cancel_event.set()
+
+    @staticmethod
+    def activate(
+        control_address: str,
+        timeout_ctrl: int,
+        socket_in_type: 'SocketType',
+        logger: 'JinaLogger',
+        **kwargs,
+    ):
+        """
+        Check if the runtime has successfully started
+
+        :param control_address: the address where the control message needs to be sent
+        :param timeout_ctrl: the timeout to wait for control messages to be processed
+        :param socket_in_type: the type of input socket, needed to know if is a dealer
+        :param logger: the JinaLogger to log messages
+        :param kwargs: extra keyword arguments
+        """
+
+        def _retry_control_message(
+            ctrl_address: str,
+            timeout_ctrl: int,
+            command: str,
+            num_retry: int,
+            logger: 'JinaLogger',
+        ):
+            from ...zmq import send_ctrl_message
+
+            for retry in range(1, num_retry + 1):
+                logger.debug(f'Sending {command} command for the {retry}th time')
+                try:
+                    send_ctrl_message(
+                        ctrl_address,
+                        command,
+                        timeout=timeout_ctrl,
+                        raise_exception=True,
+                    )
+                    break
+                except Exception as ex:
+                    logger.warning(f'{ex!r}')
+                    if retry == num_retry:
+                        raise ex
+
+        if socket_in_type == SocketType.DEALER_CONNECT:
+            _retry_control_message(
+                ctrl_address=control_address,
+                timeout_ctrl=timeout_ctrl,
+                command='ACTIVATE',
+                num_retry=3,
+                logger=logger,
+            )
+
+    @staticmethod
+    def get_control_address(host: str, port: str, **kwargs):
+        """
+        Get the control address for a runtime with a given host and port
+
+        :param host: the host where the runtime works
+        :param port: the control port where the runtime listens
+        :param kwargs: extra keyword arguments
+        :return: The corresponding control address
+        """
+        from ...zmq import Zmqlet
+
+        return Zmqlet.get_ctrl_address(host, port, False)[0]
