@@ -5,7 +5,6 @@ import mimetypes
 import os
 import urllib.parse
 import urllib.request
-import warnings
 from hashlib import blake2b
 from typing import (
     Iterable,
@@ -17,6 +16,7 @@ from typing import (
     Tuple,
     List,
     Type,
+    overload,
 )
 
 import numpy as np
@@ -36,7 +36,6 @@ from ...helper import (
     random_identity,
     download_mermaid_url,
     dunder_get,
-    cached_property,
 )
 from ...importer import ImportExtensions
 from ...logging.predefined import default_logger
@@ -86,7 +85,7 @@ DocumentSourceType = TypeVar(
 
 _all_mime_types = set(mimetypes.types_map.values())
 
-_all_doc_content_keys = {'content', 'uri', 'blob', 'text', 'buffer'}
+_all_doc_content_keys = {'content', 'blob', 'text', 'buffer', 'graph'}
 _all_doc_array_keys = ('blob', 'embedding')
 _special_mapped_keys = ('scores', 'evaluations')
 
@@ -144,6 +143,54 @@ class Document(ProtoTypeMixin, VersionedMixin):
     """
 
     ON_GETATTR = ['matches', 'chunks']
+    CNT = 0
+
+    # overload_inject_start_document
+    @overload
+    def __init__(
+        self,
+        *,
+        adjacency: Optional[int] = None,
+        blob: Optional[Union['ArrayType', 'jina_pb2.NdArrayProto', 'NdArray']] = None,
+        buffer: Optional[bytes] = None,
+        chunks: Optional[Iterable['Document']] = None,
+        content: Optional[DocumentContentType] = None,
+        embedding: Optional[
+            Union['ArrayType', 'jina_pb2.NdArrayProto', 'NdArray']
+        ] = None,
+        granularity: Optional[int] = None,
+        id: Optional[str] = None,
+        matches: Optional[Iterable['Document']] = None,
+        mime_type: Optional[str] = None,
+        modality: Optional[str] = None,
+        parent_id: Optional[str] = None,
+        tags: Optional[Union[Dict, StructView]] = None,
+        text: Optional[str] = None,
+        uri: Optional[str] = None,
+        weight: Optional[float] = None,
+        **kwargs,
+    ):
+        """
+        :param adjacency: the adjacency of this Document
+        :param blob: the blob content of thi Document
+        :param buffer: the buffer bytes from this document
+        :param chunks: the array of chunks of this document
+        :param content: the value of the content depending on `:meth:`content_type`
+        :param embedding: the embedding of this Document
+        :param granularity: the granularity of this Document
+        :param id: the id of this Document
+        :param matches: the array of matches attached to this document
+        :param mime_type: the mime_type of this Document
+        :param modality: the modality of the document.
+        :param parent_id: the parent id of this Document
+        :param tags: a Python dict view of the tags.
+        :param text: the text from this document content
+        :param uri: the uri of this Document
+        :param weight: the weight of the document
+        :param kwargs: other parameters to be set _after_ the document is constructed
+        """
+
+    # overload_inject_end_document
 
     def __init__(
         self,
@@ -180,13 +227,18 @@ class Document(ProtoTypeMixin, VersionedMixin):
                 assert d.tags['hello'] == 'world'  # true
                 assert d.tags['good'] == 'bye'  # true
         """
-        self._pb_body = jina_pb2.DocumentProto()
+        Document.CNT += 1
+
         try:
             if isinstance(document, jina_pb2.DocumentProto):
                 if copy:
+                    self._pb_body = jina_pb2.DocumentProto()
                     self._pb_body.CopyFrom(document)
                 else:
                     self._pb_body = document
+            elif isinstance(document, bytes):
+                self._pb_body = jina_pb2.DocumentProto()
+                self._pb_body.ParseFromString(document)
             elif isinstance(document, (dict, str)):
                 if isinstance(document, str):
                     document = json.loads(document)
@@ -211,13 +263,15 @@ class Document(ProtoTypeMixin, VersionedMixin):
                         field_resolver.get(k, k): v for k, v in document.items()
                     }
 
-                user_fields = set(document.keys())
+                user_fields = set(document)
                 support_fields = set(
                     self.attributes(
-                        include_proto_fields_camelcase=True, include_properties=False
+                        include_proto_fields_camelcase=True,
+                        include_properties=False,
                     )
                 )
 
+                self._pb_body = jina_pb2.DocumentProto()
                 if support_fields.issuperset(user_fields):
                     json_format.ParseDict(document, self._pb_body)
                 else:
@@ -243,29 +297,18 @@ class Document(ProtoTypeMixin, VersionedMixin):
                             self._pb_body.tags.update(
                                 {k: document[k] for k in _remainder}
                             )
-            elif isinstance(document, bytes):
-                # directly parsing from binary string gives large false-positive
-                # fortunately protobuf throws a warning when the parsing seems go wrong
-                # the context manager below converts this warning into exception and throw it
-                # properly
-                with warnings.catch_warnings():
-                    warnings.filterwarnings(
-                        'error', 'Unexpected end-group tag', category=RuntimeWarning
-                    )
-                    try:
-                        self._pb_body.ParseFromString(document)
-                    except RuntimeWarning as ex:
-                        raise BadDocType(
-                            f'fail to construct a document from {document}'
-                        ) from ex
             elif isinstance(document, Document):
                 if copy:
+                    self._pb_body = jina_pb2.DocumentProto()
                     self._pb_body.CopyFrom(document.proto)
                 else:
                     self._pb_body = document.proto
             elif document is not None:
                 # note ``None`` is not considered as a bad type
                 raise ValueError(f'{typename(document)} is not recognizable')
+            else:
+                # create an empty document
+                self._pb_body = jina_pb2.DocumentProto()
         except Exception as ex:
             raise BadDocType(
                 f'fail to construct a document from {document}, '
@@ -276,12 +319,14 @@ class Document(ProtoTypeMixin, VersionedMixin):
         if self._pb_body.id is None or not self._pb_body.id:
             self.id = random_identity(use_uuid1=True)
 
-        # check if there are mutually exclusive content fields
-        if _contains_conflicting_content(**kwargs):
-            raise ValueError(
-                f'Document content fields are mutually exclusive, please provide only one of {_all_doc_content_keys}'
-            )
-        self.set_attributes(**kwargs)
+        if kwargs:
+            # check if there are mutually exclusive content fields
+            if len(_all_doc_content_keys.intersection(kwargs.keys())) > 1:
+                raise ValueError(
+                    f'Document content fields are mutually exclusive, please provide only one of {_all_doc_content_keys}'
+                )
+            self.set_attributes(**kwargs)
+        self._mermaid_id = str(Document.CNT) + self._pb_body.id
 
     def pop(self, *fields) -> None:
         """Remove the values from the given fields of this Document.
@@ -377,7 +422,7 @@ class Document(ProtoTypeMixin, VersionedMixin):
             if (
                 field == 'tags'
             ):  # For the tags, stay consistent with the python update method.
-                self._pb_body.tags.update(source.tags)
+                self.tags.update(source.tags)
             else:
                 self._pb_body.ClearField(field)
                 try:
@@ -415,35 +460,33 @@ class Document(ProtoTypeMixin, VersionedMixin):
 
     @property
     def id(self) -> str:
-        """The document id in hex string, for non-binary environment such as HTTP, CLI, HTML and also human-readable.
-        it will be used as the major view.
+        """The document id in string.
 
-        :return: the id from the proto
+        :return: the id of this Document
         """
         return self._pb_body.id
 
     @property
     def parent_id(self) -> str:
-        """The document's parent id in hex string, for non-binary environment such as HTTP, CLI, HTML and also human-readable.
-        it will be used as the major view.
+        """The document's parent id in string.
 
-        :return: the parent id from the proto
+        :return: the parent id of this Document
         """
         return self._pb_body.parent_id
 
     @id.setter
-    def id(self, value: Union[bytes, str, int]):
+    def id(self, value: str):
         """Set document id to a string value.
 
-        :param value: id as bytes, int or str
+        :param value: id as string
         """
         self._pb_body.id = str(value)
 
     @parent_id.setter
-    def parent_id(self, value: Union[bytes, str, int]):
+    def parent_id(self, value: str):
         """Set document's parent id to a string value.
 
-        :param value: id as bytes, int or str
+        :param value: id as string
         """
         self._pb_body.parent_id = str(value)
 
@@ -458,7 +501,7 @@ class Document(ProtoTypeMixin, VersionedMixin):
             proto instance stored. In the case where the `blob` stored is sparse, it will return them as a `coo` matrix.
             If any other type of `sparse` type is desired, use the `:meth:`get_sparse_blob`.
 
-        :return: the blob content from the proto
+        :return: the blob content of thi Document
         """
         return NdArray(self._pb_body.blob).value
 
@@ -471,7 +514,7 @@ class Document(ProtoTypeMixin, VersionedMixin):
         :param kwargs: Additional key value argument, for `scipy` backend, we need to set
             the keyword `sp_format` as one of the scipy supported sparse format, such as `coo`
             or `csr`.
-        :return: the blob from the proto as an sparse array
+        :return: the blob of this Document but as an sparse array
         """
         return NdArray(
             self._pb_body.blob,
@@ -497,7 +540,7 @@ class Document(ProtoTypeMixin, VersionedMixin):
             proto instance stored. In the case where the `embedding` stored is sparse, it will return them as a `coo` matrix.
             If any other type of `sparse` type is desired, use the `:meth:`get_sparse_embedding`.
 
-        :return: the embedding from the proto
+        :return: the embedding of this Document
         """
         return NdArray(self._pb_body.embedding).value
 
@@ -510,7 +553,7 @@ class Document(ProtoTypeMixin, VersionedMixin):
         :param kwargs: Additional key value argument, for `scipy` backend, we need to set
             the keyword `sp_format` as one of the scipy supported sparse format, such as `coo`
             or `csr`.
-        :return: the embedding from the proto as an sparse array
+        :return: the embedding of this Document but as as an sparse array
         """
         return NdArray(
             self._pb_body.embedding,
@@ -772,7 +815,7 @@ class Document(ProtoTypeMixin, VersionedMixin):
     def uri(self) -> str:
         """Return the URI of the document.
 
-        :return: the uri from this document proto
+        :return: the uri of this Document
         """
         return self._pb_body.uri
 
@@ -794,7 +837,7 @@ class Document(ProtoTypeMixin, VersionedMixin):
     def mime_type(self) -> str:
         """Get MIME type of the document
 
-        :return: the mime_type from this document proto
+        :return: the mime_type of this Document
         """
         return self._pb_body.mime_type
 
@@ -822,7 +865,7 @@ class Document(ProtoTypeMixin, VersionedMixin):
     def content_type(self) -> str:
         """Return the content type of the document, possible values: text, blob, buffer
 
-        :return: the type of content present in this document proto
+        :return: the type of content of this Document
         """
         return self._pb_body.WhichOneof('content')
 
@@ -852,10 +895,7 @@ class Document(ProtoTypeMixin, VersionedMixin):
         if isinstance(value, bytes):
             self.buffer = value
         elif isinstance(value, str):
-            if _is_uri(value):
-                self.uri = value
-            else:
-                self.text = value
+            self.text = value
         elif isinstance(value, np.ndarray):
             self.blob = value
         else:
@@ -870,7 +910,7 @@ class Document(ProtoTypeMixin, VersionedMixin):
     def granularity(self):
         """Return the granularity of the document.
 
-        :return: the granularity from this document proto
+        :return: the granularity of this Document
         """
         return self._pb_body.granularity
 
@@ -886,7 +926,7 @@ class Document(ProtoTypeMixin, VersionedMixin):
     def adjacency(self):
         """Return the adjacency of the document.
 
-        :return: the adjacency from this document proto
+        :return: the adjacency of this Document
         """
         return self._pb_body.adjacency
 
@@ -966,14 +1006,23 @@ class Document(ProtoTypeMixin, VersionedMixin):
         self.blob = to_image_blob(io.BytesIO(self.buffer), color_axis)
 
     def convert_image_blob_to_uri(
-        self, width: int, height: int, resize_method: str = 'BILINEAR'
+        self,
+        width: Optional[int] = None,
+        height: Optional[int] = None,
+        resize_method: str = 'BILINEAR',
+        color_axis: int = -1,
     ):
         """Assuming :attr:`blob` is a _valid_ image, set :attr:`uri` accordingly
-        :param width: the width of the blob
-        :param height: the height of the blob
+        :param width: the width of the blob, if None, interpret from :attr:`blob` shape.
+        :param height: the height of the blob, if None, interpret from :attr:`blob` shape.
         :param resize_method: the resize method name
+        :param color_axis: the axis id of the color channel, ``-1`` indicates the color channel info at the last axis
+
+        ..note::
+            if both :attr:`width` and :attr:`height` were provided, will not resize. Otherwise, will get image size
+            by :attr:`self.blob` shape and apply resize method :attr:`resize_method`.
         """
-        png_bytes = png_to_buffer(self.blob, width, height, resize_method)
+        png_bytes = png_to_buffer(self.blob, width, height, resize_method, color_axis)
         self.uri = 'data:image/png;base64,' + base64.b64encode(png_bytes).decode()
 
     def convert_image_uri_to_blob(
@@ -1147,10 +1196,10 @@ class Document(ProtoTypeMixin, VersionedMixin):
 
         mermaid_str = (
             """
-                                %%{init: {'theme': 'base', 'themeVariables': { 'primaryColor': '#FFC666'}}}%%
-                                classDiagram
-
-                                        """
+                                    %%{init: {'theme': 'base', 'themeVariables': { 'primaryColor': '#FFC666'}}}%%
+                                    classDiagram
+    
+                                            """
             + self.__mermaid_str__()
         )
 
@@ -1294,14 +1343,6 @@ class Document(ProtoTypeMixin, VersionedMixin):
             raise AttributeError
         return value
 
-    @cached_property
-    def _mermaid_id(self) -> str:
-        """A unique ID for mermaid visualize id
-
-        :return: unique ID
-        """
-        return random_identity()
-
 
 def _is_uri(value: str) -> bool:
     scheme = urllib.parse.urlparse(value).scheme
@@ -1316,14 +1357,3 @@ def _is_uri(value: str) -> bool:
 def _is_datauri(value: str) -> bool:
     scheme = urllib.parse.urlparse(value).scheme
     return scheme in {'data'}
-
-
-def _contains_conflicting_content(**kwargs):
-    content_keys = 0
-    for k in kwargs.keys():
-        if k in _all_doc_content_keys:
-            content_keys += 1
-            if content_keys > 1:
-                return True
-
-    return False
